@@ -1,7 +1,7 @@
 package App::cryp::exchange;
 
-our $DATE = '2018-06-11'; # DATE
-our $VERSION = '0.007'; # VERSION
+our $DATE = '2018-06-13'; # DATE
+our $VERSION = '0.008'; # VERSION
 
 use 5.010001;
 use strict;
@@ -10,6 +10,14 @@ use Log::ger;
 
 our %SPEC;
 
+our %arg_req0_account = (
+    account => {
+        schema => 'cryptoexchange::account*',
+        req => 1,
+        pos => 0,
+    },
+);
+
 our %arg_detail = (
     detail => {
         schema => 'bool*',
@@ -23,11 +31,18 @@ our %arg_native = (
     },
 );
 
-our %arg_req0_account = (
-    account => {
-        schema => 'cryptoexchange::account*',
+our %arg_req3_order_id = (
+    order_id => {
+        schema => ['str*'],
         req => 1,
-        pos => 0,
+        pos => 3,
+    },
+);
+
+our %arg_1_pair = (
+    pair => {
+        schema => 'str*',
+        pos => 1,
     },
 );
 
@@ -40,6 +55,25 @@ our %arg_req1_pair = (
     },
 );
 
+our %arg_req3_price = (
+    price => {
+        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
+        req => 1,
+        pos => 3,
+    },
+);
+
+our %args_size = (
+    base_size => {
+        summary => 'Order amount, denominated in base currency (first currency of the pair)',
+        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
+    },
+    quote_size => {
+        summary => 'Order amount, denominated in quote currency (second currency of the pair)',
+        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
+    },
+);
+
 our %arg_type = (
     type => {
         schema => ['str*', in=>['buy','sell']],
@@ -63,33 +97,6 @@ our %arg_req2_type = (
     },
 );
 
-our %arg_req3_price = (
-    price => {
-        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
-        req => 1,
-        pos => 3,
-    },
-);
-
-our %args_size = (
-    base_size => {
-        summary => 'Order amount, denominated in base currency (first currency of the pair)',
-        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
-    },
-    quote_size => {
-        summary => 'Order amount, denominated in quote currency (second currency of the pair)',
-        schema => ['float*', xmin=>0, 'x.perl.coerce_rules'=>['str_num_en']],
-    },
-);
-
-our %arg_req3_order_id = (
-    order_id => {
-        schema => ['str*'],
-        req => 1,
-        pos => 3,
-    },
-);
-
 $SPEC{':package'} = {
     v => 1.1,
     summary => 'Interact with cryptoexchanges using a common interface',
@@ -119,41 +126,6 @@ sub _init {
     [200];
 }
 
-$SPEC{exchanges} = {
-    v => 1.1,
-    summary => 'List supported exchanges',
-    args => {
-        %arg_detail,
-    },
-    tags => ['category:etc'],
-};
-sub exchanges {
-    require PERLANCAR::Module::List;
-
-    my %args = @_;
-
-    my $mods = PERLANCAR::Module::List::list_modules(
-        "App::cryp::Exchange::", {list_modules=>1});
-
-    my @res;
-    for my $mod (sort keys %$mods) {
-        my ($safename) = $mod =~ /::(\w+)\z/;
-        $safename =~ s/_/-/g;
-        push @res, {
-            safename => $safename,
-        };
-    }
-
-    unless ($args{detail}) {
-        @res = map {$_->{safename}} @res;
-    }
-
-    my $resmeta = {
-    };
-
-    [200, "OK", \@res, $resmeta];
-}
-
 $SPEC{accounts} = {
     v => 1.1,
     summary => 'List exchange accounts',
@@ -192,16 +164,14 @@ sub accounts {
 
 }
 
-$SPEC{pairs} = {
+$SPEC{balance} = {
     v => 1.1,
-    summary => 'List pairs supported by exchange',
+    summary => 'List account balance',
     args => {
         %arg_req0_account,
-        %arg_detail,
-        %arg_native,
     },
 };
-sub pairs {
+sub balance {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
@@ -209,10 +179,128 @@ sub pairs {
     my $res = _init($r); return $res unless $res->[0] == 200;
     my $xchg = $r->{_stash}{exchange_client};
 
-    $xchg->list_pairs(
-        detail => $args{detail},
-        native => $args{native},
-    );
+    $xchg->list_balances;
+}
+
+$SPEC{cancel_order} = {
+    v => 1.1,
+    summary => 'Cancel an order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_order_id,
+    },
+};
+sub cancel_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->cancel_order(%args);
+}
+
+$SPEC{create_limit_order} = {
+    v => 1.1,
+    summary => 'Create a limit order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_price,
+        %args_size,
+    },
+    args_rels => {
+        req_one => [qw/base_size quote_size/],
+    },
+};
+sub create_limit_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->create_limit_order(%args);
+}
+
+$SPEC{exchanges} = {
+    v => 1.1,
+    summary => 'List supported exchanges',
+    args => {
+        %arg_detail,
+    },
+    tags => ['category:etc'],
+};
+sub exchanges {
+    require PERLANCAR::Module::List;
+
+    my %args = @_;
+
+    my $mods = PERLANCAR::Module::List::list_modules(
+        "App::cryp::Exchange::", {list_modules=>1});
+
+    my @res;
+    for my $mod (sort keys %$mods) {
+        my ($safename) = $mod =~ /::(\w+)\z/;
+        $safename =~ s/_/-/g;
+        push @res, {
+            safename => $safename,
+        };
+    }
+
+    unless ($args{detail}) {
+        @res = map {$_->{safename}} @res;
+    }
+
+    my $resmeta = {
+    };
+
+    [200, "OK", \@res, $resmeta];
+}
+
+$SPEC{get_order} = {
+    v => 1.1,
+    summary => 'Get information about an order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_order_id,
+    },
+};
+sub get_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->get_order(%args);
+}
+
+$SPEC{open_orders} = {
+    v => 1.1,
+    summary => "List open orders",
+    args => {
+        %arg_req0_account,
+        %arg_1_pair,
+    },
+};
+sub open_orders {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->list_open_orders(%args);
 }
 
 $SPEC{orderbook} = {
@@ -264,14 +352,16 @@ sub orderbook {
     [200, "OK", \@rows];
 }
 
-$SPEC{balance} = {
+$SPEC{pairs} = {
     v => 1.1,
-    summary => 'List account balance',
+    summary => 'List pairs supported by exchange',
     args => {
         %arg_req0_account,
+        %arg_detail,
+        %arg_native,
     },
 };
-sub balance {
+sub pairs {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
@@ -279,24 +369,21 @@ sub balance {
     my $res = _init($r); return $res unless $res->[0] == 200;
     my $xchg = $r->{_stash}{exchange_client};
 
-    $xchg->list_balances;
+    $xchg->list_pairs(
+        detail => $args{detail},
+        native => $args{native},
+    );
 }
 
-$SPEC{create_limit_order} = {
+$SPEC{ticker} = {
     v => 1.1,
-    summary => 'Create a limit order',
+    summary => "Get a pair's ticker (last 24h price & volume information)",
     args => {
         %arg_req0_account,
         %arg_req1_pair,
-        %arg_req2_type,
-        %arg_req3_price,
-        %args_size,
-    },
-    args_rels => {
-        req_one => [qw/base_size quote_size/],
     },
 };
-sub create_limit_order {
+sub ticker {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
@@ -304,49 +391,7 @@ sub create_limit_order {
     my $res = _init($r); return $res unless $res->[0] == 200;
     my $xchg = $r->{_stash}{exchange_client};
 
-    $xchg->create_limit_order(%args);
-}
-
-$SPEC{get_order} = {
-    v => 1.1,
-    summary => 'Get information about an order',
-    args => {
-        %arg_req0_account,
-        %arg_req1_pair,
-        %arg_req2_type,
-        %arg_req3_order_id,
-    },
-};
-sub get_order {
-    my %args = @_;
-
-    my $r = $args{-cmdline_r};
-
-    my $res = _init($r); return $res unless $res->[0] == 200;
-    my $xchg = $r->{_stash}{exchange_client};
-
-    $xchg->get_order(%args);
-}
-
-$SPEC{cancel_order} = {
-    v => 1.1,
-    summary => 'Cancel an order',
-    args => {
-        %arg_req0_account,
-        %arg_req1_pair,
-        %arg_req2_type,
-        %arg_req3_order_id,
-    },
-};
-sub cancel_order {
-    my %args = @_;
-
-    my $r = $args{-cmdline_r};
-
-    my $res = _init($r); return $res unless $res->[0] == 200;
-    my $xchg = $r->{_stash}{exchange_client};
-
-    $xchg->cancel_order(%args);
+    $xchg->get_ticker(%args);
 }
 
 1;
@@ -364,7 +409,7 @@ App::cryp::exchange - Interact with cryptoexchanges using a common interface
 
 =head1 VERSION
 
-This document describes version 0.007 of App::cryp::exchange (from Perl distribution App-cryp-exchange), released on 2018-06-11.
+This document describes version 0.008 of App::cryp::exchange (from Perl distribution App-cryp-exchange), released on 2018-06-13.
 
 =head1 SYNOPSIS
 
@@ -579,6 +624,38 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 open_orders
+
+Usage:
+
+ open_orders(%args) -> [status, msg, result, meta]
+
+List open orders.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<pair> => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 orderbook
 
 Usage:
@@ -646,6 +723,38 @@ that contains extra information.
 
 Return value:  (any)
 
+
+=head2 ticker
+
+Usage:
+
+ ticker(%args) -> [status, msg, result, meta]
+
+Get a pair's ticker (last 24h price & volume information).
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<pair>* => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-cryp-exchange>.
